_SPECIES_REGEX = r"^([A-Za-z ]+|http://purl\.obolibrary\.org/obo/NCBITaxon_\d+)$"
_ALLOWED_SEXES = {
    "male",
    "m",
    "M",
    "MALE",
    "Male",
    "female",
    "f",
    "F",
    "FEMALE",
    "Female",
    "other",
    "o",
    "O",
    "OTHER",
    "Other",
}
